"use strict";
var child_process_1 = require("child_process");
var os_1 = require("os");
function execute(cmd, args, io, cwd, _spawn) {
    if (_spawn === void 0) { _spawn = child_process_1.spawn; }
    var stdio = [
        io.stdin === process.stdin ? 'inherit' : 'pipe',
        io.stdout === process.stdout ? 'inherit' : 'pipe',
        io.stderr === process.stderr ? 'inherit' : 'pipe',
    ];
    return new Promise(function (resolve, reject) {
        var cp = _spawn(cmd, args, { cwd: cwd, stdio: stdio });
        var successBuffer = [];
        var errorBuffer = [];
        if (stdio[0] === 'pipe')
            io.stdin.pipe(cp.stdin);
        if (stdio[1] === 'pipe')
            cp.stdout.on('data', function (d) {
                successBuffer.push(d.toString());
                io.stdout.write(d);
            });
        if (stdio[2] === 'pipe')
            cp.stderr.on('data', function (d) {
                errorBuffer.push(d.toString());
                io.stderr.write(d);
            });
        var output = function () { return ({ stdout: successBuffer.join(''), stderr: errorBuffer.join('') }); };
        function resolveWithBuffer(exitCode) {
            if (typeof exitCode === 'number' && exitCode !== 0)
                reject(output());
            resolve(output());
        }
        ;
        var rejectWithBuffer = function () { return reject(output()); };
        cp.on('close', writeAndEnd(io.stdout, resolveWithBuffer));
        cp.on('end', writeAndEnd(io.stdout, resolveWithBuffer));
        cp.on('error', writeAndEnd(io.stderr, rejectWithBuffer));
    });
}
exports.execute = execute;
function writeAndEnd(writable, end) {
    return function (exitCode) {
        writable.write(os_1.EOL);
        end(exitCode);
    };
}
;
//# sourceMappingURL=execute.js.map