"use strict";
var helpers_1 = require("../../helpers");
var child_process_1 = require("child_process");
var stdio_mock_1 = require("stdio-mock");
var parseCommitMessage_1 = require("./parseCommitMessage");
exports.separator = '-----------';
exports.format = '%H' + exports.separator +
    '%h' + exports.separator +
    '%an' + exports.separator +
    '%ae' + exports.separator +
    '%ct' + exports.separator +
    '%B' + exports.separator; // raw body
function gitRawCommits(fromCommitHash, io, cwd, _spawn) {
    if (io === void 0) { io = stdio_mock_1.stdio(); }
    if (cwd === void 0) { cwd = process.cwd(); }
    if (_spawn === void 0) { _spawn = child_process_1.spawn; }
    var cmd = "git";
    var args = [
        "log",
        "--format=" + exports.format,
        fromCommitHash !== void 0 ? fromCommitHash + "..HEAD" : "",
    ].filter(Boolean);
    return helpers_1.execute(cmd, args, io, cwd, _spawn).then(function (_a) {
        var stdout = _a.stdout;
        var hashesAndMessages = stdout.split(exports.separator);
        var commits = [];
        for (var i = 0; i < hashesAndMessages.length; i = i + 6) {
            // usually end up with an extra "empty" set
            if (!hashesAndMessages[i + 5])
                break;
            var hash = hashesAndMessages[i].replace('\n', '');
            var abbreviatedHash = hashesAndMessages[i + 1];
            var authorName = hashesAndMessages[i + 2];
            var authorEmail = hashesAndMessages[i + 3];
            var time = parseInt(hashesAndMessages[i + 4]);
            var message = hashesAndMessages[i + 5];
            commits.push({
                hash: hash,
                abbreviatedHash: abbreviatedHash,
                authorEmail: authorEmail,
                authorName: authorName,
                time: time,
                message: parseCommitMessage_1.parseCommitMessage(message),
            });
        }
        return commits;
    });
}
exports.gitRawCommits = gitRawCommits;
;
//# sourceMappingURL=gitRawCommits.js.map