"use strict";
var os_1 = require("os");
// Hackish super dependent upon the way that commit plugin works
function parseCommitMessage(rawCommit) {
    if (rawCommit.trim().startsWith('Merge')) {
        return {
            type: 'merge',
            scope: null,
            subject: '',
            body: '',
            affects: null,
            breakingChanges: null,
            issuesClosed: null,
            suggestedUpdate: 0,
            raw: rawCommit,
        };
    }
    try {
        var type = rawCommit.split('(')[0].trim();
        var scope = (rawCommit.split('(')[1] || '').split('):')[0].trim();
        var subject = (rawCommit.split('):') || '')[1].split(os_1.EOL)[0];
        var messageBody = rawCommit.split(subject)[1] || '';
        var body = messageBody
            .split('AFFECTS')[0] // ensure doesn't contain affects
            .split('BREAKING')[0] // ensure doesn't contain breaking changes
            .split('ISSUES')[0] // ensure doesn't contain the issues closed
            .replace(new RegExp("[" + os_1.EOL + "]{2,}", 'g'), os_1.EOL).trim();
        var affects = getAffects(messageBody);
        var breakingChanges = getBreakingChanges(messageBody);
        var issuesClosed = getIssuesClosed(messageBody);
        var suggestedUpdate = getSuggestedUpdate(type, breakingChanges);
        return {
            type: type,
            scope: scope,
            subject: subject.trim(),
            body: body,
            affects: affects,
            breakingChanges: breakingChanges,
            issuesClosed: issuesClosed,
            suggestedUpdate: suggestedUpdate,
            raw: rawCommit,
        };
    }
    catch (e) {
        return {
            type: '',
            scope: '',
            subject: '',
            body: '',
            affects: null,
            breakingChanges: null,
            issuesClosed: null,
            suggestedUpdate: 0,
            raw: rawCommit,
        };
    }
}
exports.parseCommitMessage = parseCommitMessage;
function splitEndOfLine(str) {
    str = str.trim();
    if (str.indexOf("\r\n"))
        return str.split("\r\n" + "\r\n")[0];
    return str.split(os_1.EOL + os_1.EOL)[0];
}
function getAffects(messageBody) {
    var affects = messageBody.split('AFFECTS:')[1];
    if (!affects)
        return null;
    return affects
        .split('BREAKING')[0] // ensure doesn't contain breaking changes
        .split('ISSUES')[0] // ensure doesn't contain the issues closed
        .split(os_1.EOL + os_1.EOL)[0]
        .trim()
        .split(',')
        .map(splitEndOfLine);
}
function getBreakingChanges(messageBody) {
    var breakingChanges = messageBody.split(/BREAKING[\sA-Z]+:/)[1];
    if (!breakingChanges)
        return null;
    return breakingChanges.split(os_1.EOL + os_1.EOL)[0].trim();
}
function getIssuesClosed(messageBody) {
    var issuesClosed = messageBody.split(/ISSUES[\sA-Z]+:/)[1];
    if (!issuesClosed)
        return null;
    return issuesClosed.trim().split(',').map(function (str) { return str.trim(); });
}
/*
 * Suggested Updates Using Semver
 * 0: none
 * 1: patch
 * 2: minor
 * 3: major
 */
function getSuggestedUpdate(type, breakingChanges) {
    if (breakingChanges !== null)
        return 3;
    if (type === 'feat')
        return 2;
    if (type === 'fix' || type === 'perf')
        return 1;
    return 0;
}
//# sourceMappingURL=parseCommitMessage.js.map