"use strict";
var helpers_1 = require("../../helpers");
var os_1 = require("os");
var sequence_1 = require("@typed/sequence");
function each(command, callback) {
    return new Promise(function (resolve, reject) {
        command.handler = function (input, io) {
            var config = input.config, options = input.options;
            var call = createCallback(callback, input, io);
            var packagesToExec = options.changed
                ? helpers_1.getChangedPackages(config.packages)
                : Promise.resolve(helpers_1.packagesToExecute(input));
            return packagesToExec
                .then(function (packages) {
                if (packages.length === 0)
                    return io.stdout.write("No packages could be found" + os_1.EOL);
                return sequence_1.sequence(packages, call);
            })
                .then(resolve)
                .catch(reject);
        };
    }).catch(function (e) {
        if (e.stderr) {
            process.stderr.write(e.stderr + os_1.EOL);
        }
        else {
            process.stderr.write((e.message || e) + os_1.EOL);
        }
        process.exit(1);
    });
}
exports.each = each;
function createCallback(callback, input, stdio) {
    return function (pkg) {
        var handlerOptions = Object.assign({}, input, { pkg: pkg, args: input.args.slice(0) });
        return callback(handlerOptions, stdio);
    };
}
//# sourceMappingURL=each.js.map