"use strict";
var os_1 = require("os");
var path_1 = require("path");
var typed_colors_1 = require("typed-colors");
var findup = require("findup-sync");
var NORTHBROOK_CONFIG = ['northbrook.ts', 'northbrook.js'];
/**
 * Find the path of a northbrook.js file and require it.
 *
 * @export
 * @param {STDIO} [stdio]
 * @returns {({ path: string, config: NorthbrookConfig } | null)}
 */
function findNorthbrookConfig(debug, stdio, options) {
    if (debug === void 0) { debug = false; }
    if (stdio === void 0) { stdio = {}; }
    var _a = stdio.stdout, stdout = _a === void 0 ? process.stdout : _a;
    var northbrookFilePath = findup(NORTHBROOK_CONFIG, options);
    if (!northbrookFilePath) {
        if (debug)
            stdout.write(typed_colors_1.yellow(typed_colors_1.bold("WARNING")) + ": "
                + typed_colors_1.bold("Failed to find a Northbrook configuration file")
                + os_1.EOL);
        return { path: null, config: null };
    }
    var config = require(northbrookFilePath);
    var path = path_1.dirname(northbrookFilePath);
    return { path: path, config: config };
}
exports.findNorthbrookConfig = findNorthbrookConfig;
//# sourceMappingURL=findNorthbrookConfig.js.map