"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var os_1 = require("os");
var path_1 = require("path");
var reginn_1 = require("reginn");
var ramda_1 = require("ramda");
var typed_colors_1 = require("typed-colors");
var resolvePlugins_1 = require("./resolvePlugins");
var resolvePackages_1 = require("./resolvePackages");
var run_1 = require("./run");
var buildDependencyGraph_1 = require("./buildDependencyGraph");
var helpers_1 = require("../../helpers");
var only = reginn_1.flag('string', reginn_1.alias('only', 'o'), reginn_1.description('Execute plugins in only specific packages'));
var changed = reginn_1.flag('boolean', reginn_1.alias('changed', 'c'), reginn_1.description('Execute plugins in only packages that have changed since last release'));
var configPath = reginn_1.flag('string', reginn_1.alias('config', 'c'), reginn_1.description('Relative path to your northbrook config'));
var debug = reginn_1.flag('boolean', reginn_1.alias('debug', 'd'), reginn_1.description('Log more information for debugging'));
// for display in help menu
var nb = reginn_1.app(only, changed, configPath, debug);
function northbrook(config, additionalPlugins, cwd, stdio, debugMode) {
    if (additionalPlugins === void 0) { additionalPlugins = []; }
    if (cwd === void 0) { cwd = process.cwd(); }
    if (debugMode === void 0) { debugMode = false; }
    process.chdir(cwd);
    process.env.PATH = path_1.join(cwd, 'node_modules/.bin/') + path_1.delimiter + process.env.PATH;
    stdio = __assign({}, helpers_1.defaultStdio, stdio || {});
    var plugins = resolvePlugins_1.resolvePlugins(config.plugins || [], cwd, stdio, debugMode)
        .concat(additionalPlugins);
    var packages = resolvePackages_1.resolvePackages(config.packages || [], cwd, stdio, debugMode);
    if (packages.length === 0)
        packages = helpers_1.isFile(path_1.join(cwd, 'package.json')) ? [cwd] : [];
    var dependencyGraph = buildDependencyGraph_1.buildDependencyGraph(packages, config.circular);
    config.packages = dependencyGraph.paths();
    var run = run_1.northrookRun(ramda_1.clone(config), dependencyGraph, cwd, stdio);
    return {
        plugins: plugins.slice(0),
        packages: packages.slice(0),
        start: function (argv) {
            argv = argv || process.argv.slice(2);
            if (debugMode) {
                stdio.stdout.write(typed_colors_1.cyan("DEBUG") +
                    (": Starting northbrook with args " + argv.join(' ')) + os_1.EOL + os_1.EOL);
            }
            return run(argv, reginn_1.app.apply(void 0, [nb].concat(plugins.map(ramda_1.prop('plugin')))));
        },
    };
}
exports.northbrook = northbrook;
//# sourceMappingURL=northbrook.js.map