"use strict";
var os_1 = require("os");
var path_1 = require("path");
var fs_1 = require("fs");
var ramda_1 = require("ramda");
var typed_colors_1 = require("typed-colors");
var helpers_1 = require("../../../helpers");
// finds all require()-able packages
function resolvePackages(packages, cwd, stdio, debug) {
    if (cwd === void 0) { cwd = process.cwd(); }
    if (debug === void 0) { debug = false; }
    return ramda_1.filter(Boolean, ramda_1.flatten(ramda_1.map(resolvePackage(cwd, stdio, debug), packages)));
}
exports.resolvePackages = resolvePackages;
function resolvePackage(cwd, stdio, debug) {
    return function (packagePath) {
        // allow shorthand for directories with lots of packages
        if (packagePath.endsWith('*') || packagePath.endsWith('**')) {
            var packagesPath_1 = path_1.join(cwd, packagePath.replace(/(\*)+$/, ''));
            var packages = getAllDirectories(packagesPath_1);
            return ramda_1.map(function (name) { return resolve(path_1.join(packagesPath_1, name), stdio, debug); }, packages);
        }
        return resolve(path_1.join(cwd, packagePath), stdio, debug);
    };
}
// if a directory contains a package.json
function resolve(path, stdio, debug) {
    if (helpers_1.isFile(path))
        return null;
    if (hasPkg(path)) {
        if (debug)
            stdio.stdout.write(typed_colors_1.cyan('DEBUG') + ": Resolved package: " + path + os_1.EOL);
        return path;
    }
    ;
    stdio.stdout.write(typed_colors_1.yellow("WARNING") + (": Could not resolve package: " + path) + os_1.EOL);
    return null;
}
function hasPkg(path) {
    return helpers_1.isFile(path_1.join(path, 'package.json'));
}
function getAllDirectories(rootDirectory) {
    return fs_1.readdirSync(rootDirectory);
}
//# sourceMappingURL=resolvePackages.js.map