"use strict";
var os_1 = require("os");
var path_1 = require("path");
var ramda_1 = require("ramda");
var typed_colors_1 = require("typed-colors");
var tryRequire_1 = require("./tryRequire");
// finds all require()-able plugins
function resolvePlugins(plugins, cwd, stdio, debug) {
    if (cwd === void 0) { cwd = process.cwd(); }
    if (debug === void 0) { debug = false; }
    return ramda_1.filter(Boolean, ramda_1.flatten(ramda_1.map(resolvePlugin(cwd, stdio, debug), plugins)));
}
exports.resolvePlugins = resolvePlugins;
var NORTHBROOK_PREFIXES = ['', '@northbrook/', 'northbrook-'];
function resolvePlugin(cwd, stdio, debug) {
    return function (pluginName) {
        if (isCommandOrApp(pluginName))
            return { plugin: pluginName };
        var plugin = tryRequire_1.tryRequire(path_1.join(cwd, pluginName), cwd, debug);
        if (isPlugin(plugin))
            return plugin;
        for (var i = 0; i < NORTHBROOK_PREFIXES.length; ++i) {
            plugin = tryRequire_1.tryRequire(NORTHBROOK_PREFIXES[i] + pluginName, cwd, debug);
            if (isPlugin(plugin)) {
                if (debug)
                    stdio.stdout.write(typed_colors_1.cyan('DEBUG') + (": Resolved plugin " + pluginName) + os_1.EOL);
                return plugin;
            }
        }
        stdio.stdout.write(typed_colors_1.yellow("WARNING") + (": Could not resolve plugin: " + pluginName) + os_1.EOL);
        return null;
    };
}
function isPlugin(plugin) {
    if (plugin instanceof Error)
        return false;
    if (!plugin || !plugin.plugin)
        return false;
    return isCommandOrApp(plugin.plugin);
}
function isCommandOrApp(x) {
    if (x.type === 'command' || x.type === 'app')
        return true;
    return false;
}
//# sourceMappingURL=resolvePlugins.js.map