"use strict";
var getCommandFlags_1 = require("reginn/lib/commonjs/run/getCommandFlags");
var ramda_1 = require("ramda");
var helpers_1 = require("../../../helpers");
function callCommand(argv, parsedArgs, flags, filter, config, directory, stdio, depGraph) {
    return function (command) {
        if (!command.handler)
            return;
        if (command.commands.length > 0) {
            var commandFlags = helpers_1.deepMerge(flags, getCommandFlags_1.getCommandFlags(command.commands));
            command.handler(createSubApplication(argv, parsedArgs, commandFlags, command, filter, config, directory, depGraph), stdio);
        }
        else if (command.aliases && command.aliases.length > 0) {
            command.handler({
                config: config,
                directory: directory,
                depGraph: depGraph,
                args: ramda_1.tail(parsedArgs),
                options: optionsToCamelCase(filter(command)),
            }, stdio);
        }
        else {
            command.handler({
                config: config,
                directory: directory,
                depGraph: depGraph,
                args: parsedArgs,
                options: optionsToCamelCase(filter(command)),
            }, stdio);
        }
    };
}
exports.callCommand = callCommand;
function optionsToCamelCase(options) {
    return Object.keys(options).reduce(function (acc, key) {
        var value = options[key];
        acc[toCamelCase(key)] = value;
        return acc;
    }, {});
}
function toCamelCase(str) {
    return str.replace(/-\w/, function (x) { return x[1].toUpperCase(); });
}
function createSubApplication(argv, parsedArgs, commandFlags, command, filter, config, directory, depGraph) {
    var flags = argv.filter(function (arg) { return parsedArgs.indexOf(arg) === -1; });
    var args = ramda_1.tail(parsedArgs).concat(flags);
    return {
        type: 'app',
        args: args,
        options: optionsToCamelCase(filter(command)),
        commands: command.commands,
        flags: commandFlags,
        config: config,
        directory: directory,
        depGraph: depGraph,
    };
}
//# sourceMappingURL=callCommand.js.map