"use strict";
var typed_colors_1 = require("typed-colors");
var display_1 = require("./display");
var ramda_1 = require("ramda");
// avoid reimplementing everything
var parseArguments_1 = require("reginn/lib/commonjs/run/parseArguments");
var os_1 = require("os");
var callCommand_1 = require("./callCommand");
var typed_figures_1 = require("typed-figures");
var helpers_1 = require("../../../helpers");
var filterOptions_1 = require("reginn/lib/commonjs/run/filterOptions");
var getCommandFlags_1 = require("reginn/lib/commonjs/run/getCommandFlags");
var matchCommands_1 = require("reginn/lib/commonjs/run/matchCommands");
function northrookRun(config, depGraph, directory, stdio) {
    return function run(argv, app) {
        var stdout = stdio.stdout;
        var parsedArguments = parseArguments_1.parseArguments(argv, app.flags);
        // show help if no arguments are passed in
        if (argv.length === 0) {
            stdout.write(typed_colors_1.bold("No commands were expressed..." + os_1.EOL + os_1.EOL));
            parsedArguments.help = true;
        }
        var matchedCommands = matchCommands_1.matchCommands(app, parsedArguments);
        // fail early if no commands have been matched
        if (matchedCommands.length === 0 && !parsedArguments.help) {
            throw new Error(typed_colors_1.red(typed_colors_1.bold(typed_figures_1.cross)) + ' ' +
                typed_colors_1.white('Can not find command ') +
                (parsedArguments._[0] ? typed_colors_1.red(typed_colors_1.bold("" + parsedArguments._[0])) : ''));
        }
        return execute(argv, app, config, depGraph, directory, stdio, matchedCommands, parsedArguments);
    };
}
exports.northrookRun = northrookRun;
function execute(argv, app, config, depGraph, directory, stdio, matchedCommands, parsedArguments) {
    var _a = stdio.stdout, stdout = _a === void 0 ? process.stdout : _a;
    var commandFlags = helpers_1.deepMerge(app.flags, getCommandFlags_1.getCommandFlags(matchedCommands));
    var _b = parseArguments_1.splitArguments(parseArguments_1.parseArguments(argv, commandFlags)), args = _b[0], options = _b[1];
    var filterCommandOptions = filterOptions_1.filterOptions(options, app.flags, argv);
    var commandCall = callCommand_1.callCommand(argv, args, commandFlags, filterCommandOptions, config, directory, stdio, depGraph);
    if (parsedArguments.help === true) {
        stdout.write(typed_colors_1.green(typed_colors_1.bold("Northbrook")) + os_1.EOL + os_1.EOL +
            ("" + display_1.displayFlags(app.flags).replace(/,--/, '--')) + os_1.EOL + os_1.EOL +
            ("" + app.commands.map(display_1.display))
                .replace(new RegExp(os_1.EOL + ",", 'g'), os_1.EOL)
                .replace(new RegExp("[" + os_1.EOL + "]{3,}", 'g'), os_1.EOL + os_1.EOL)
                .trim() + os_1.EOL + os_1.EOL);
    }
    else {
        // call all matched commands
        ramda_1.forEach(ramda_1.ifElse(hasHandlerFn, commandCall, logWarning(stdout)), matchedCommands);
    }
    return Promise.resolve({
        config: config,
        depGraph: depGraph,
        directory: directory,
        type: 'app',
        flags: commandFlags,
        commands: matchedCommands,
        args: args,
        options: options,
    });
}
function hasHandlerFn(command) {
    return isFunction(command.handler);
}
function isFunction(x) {
    return !!(x && x.constructor && x.call && x.apply);
}
function logWarning(stdout) {
    return function (command) {
        var aliases = command.aliases;
        if (aliases.length > 0) {
            var name_1 = aliases[0].name;
            stdout.write(typed_colors_1.yellow("" + name_1) + typed_colors_1.white(" does not have an associated handler!" + os_1.EOL));
        }
    };
}
//# sourceMappingURL=run.js.map