"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var os_1 = require("os");
var stdio_mock_1 = require("stdio-mock");
var sequence_1 = require("@typed/sequence");
var typed_prompts_1 = require("typed-prompts");
var typed_colors_1 = require("typed-colors");
var helpers_1 = require("../../../helpers");
var splitVersion_1 = require("./splitVersion");
var incrementName_1 = require("./incrementName");
var getNewVersion_1 = require("./getNewVersion");
var getPackagesToUpdate_1 = require("./getPackagesToUpdate");
function bumpPackageVersions(config, io, options, affectedPackages) {
    var packages = config.packages;
    var affectedPackageNames = Object.keys(affectedPackages);
    var packagesToUpdate = getPackagesToUpdate_1.getPackagesToUpdate(packages, affectedPackageNames);
    var method = options.semver ? 'semver' : options.comver ? 'comver' : 'semver';
    return performVersionBump(packagesToUpdate, affectedPackages, io, method);
}
exports.bumpPackageVersions = bumpPackageVersions;
function performVersionBump(packagesToUpdate, affectedPackages, io, method) {
    var newPackages = [];
    return sequence_1.sequence(packagesToUpdate, bumpVersion(affectedPackages, io, method, newPackages))
        .then(function () { return newPackages.filter(Boolean); });
}
;
var choices = ["1.0.0", "0.1.0", "0.0.1"];
function bumpVersion(affectedPackages, io, method, newPackages) {
    return function (packageToUpdate) {
        var pkg = packageToUpdate.pkg, name = packageToUpdate.name, directory = packageToUpdate.directory;
        var commits = affectedPackages[name].commits;
        var suggestedUpdate = helpers_1.getSuggestedUpdate(commits, method);
        var increment = incrementName_1.incrementName(suggestedUpdate);
        if (!increment)
            return Promise.resolve(void 0);
        var args = ['version', '--no-git-tag-version'];
        var versionQuestion = typed_prompts_1.list("version", "What version would you like to start with? ", choices, {
            default: "1.0.0",
        });
        return helpers_1.execute('npm', ['dist-tag', 'ls', name], stdio_mock_1.stdio(), directory)
            .then(function () {
            var newVersion = getSuggestedVersion(pkg.version, suggestedUpdate);
            logUpdate(pkg.version, newVersion, name, io);
            generateNewPkg(pkg, newVersion, name, directory, commits, newPackages);
            var message = generateCommitMessage(newVersion, name);
            return helpers_1.execute('npm', args.concat(increment), io, directory)
                .then(function () { return helpers_1.execute('git', ['add', 'package.json'], io, directory); })
                .then(function () { return helpers_1.execute('git', ['commit', '-m', message], io, directory); });
        })
            .catch(function () {
            io.stdout.write(typed_colors_1.bold(typed_colors_1.green("No previous versions for " + name + " could be found...")) + os_1.EOL);
            return typed_prompts_1.prompt([versionQuestion])
                .then(function (_a) {
                var newVersion = _a.version;
                if (pkg.version === newVersion)
                    return Promise.resolve(void 0);
                generateNewPkg(pkg, newVersion, name, directory, commits, newPackages);
                var message = generateCommitMessage(newVersion, name);
                return helpers_1.execute('npm', args.concat(newVersion), io, directory)
                    .then(function () { return helpers_1.execute('git', ['add', 'package.json'], io, directory); })
                    .then(function () { return helpers_1.execute('git', ['commit', '-m', message], io, directory); });
            });
        });
    };
}
;
function logUpdate(previousVersion, newVersion, name, io) {
    io.stdout.write("Bumping " + name + " version from " + previousVersion + " to " + newVersion + "..." + os_1.EOL + os_1.EOL);
}
function getSuggestedVersion(version, suggestedUpdate) {
    return getNewVersion_1.getNewVersion(splitVersion_1.splitVersion(version), suggestedUpdate);
}
function generateCommitMessage(version, name) {
    return "chore(release): v" + version + " [ci skip]" + os_1.EOL + os_1.EOL + ("AFFECTS: " + name);
}
function generateNewPkg(pkg, newVersion, name, directory, commits, newPackages) {
    var newPackage = Object.assign({}, { name: name, directory: directory, commits: commits, pkg: __assign({}, pkg, { version: newVersion }) });
    newPackages.push(newPackage);
}
//# sourceMappingURL=bumpPackageVersions.js.map