"use strict";
var child_process_1 = require("child_process");
var stdio_mock_1 = require("stdio-mock");
var os_1 = require("os");
var path_1 = require("path");
var fs_1 = require("fs");
var helpers_1 = require("../../../helpers");
var commitMessage = 'docs(CHANGELOG): append to changelog [skip ci]';
function generateChangelog(releasePackage, io, _spawn, writeStream) {
    if (io === void 0) { io = stdio_mock_1.stdio(); }
    if (_spawn === void 0) { _spawn = child_process_1.spawn; }
    var directory = releasePackage.directory, name = releasePackage.name;
    var CHANGELOG = path_1.join(directory, 'CHANGELOG.md');
    var fileContents = helpers_1.exists(CHANGELOG) ? fs_1.readFileSync(CHANGELOG).toString() : os_1.EOL;
    var writeFileSream = writeStream || fs_1.createWriteStream(CHANGELOG);
    io.stdout.write("Generating changelog for " + name + "..." + os_1.EOL + os_1.EOL);
    return writeChangelog(releasePackage, fileContents, writeFileSream)
        .then(function () { return helpers_1.execute('git', ['add', 'CHANGELOG.md'], io, directory, _spawn); })
        .then(function () { return helpers_1.execute('git', ['commit', '-m', commitMessage], io, directory, _spawn); });
}
exports.generateChangelog = generateChangelog;
function writeChangelog(releasePackage, fileContents, changelog) {
    return new Promise(function (resolve, reject) {
        var _a = releasePackage.pkg, version = _a.version, _b = _a.bugs, bugs = _b === void 0 ? { url: '' } : _b;
        var sections = {
            breaks: [],
            feat: [],
            fix: [],
            perf: [],
        };
        var titles = {
            breaks: "Breaking Changes",
            feat: "Features",
            fix: "Bug Fixes",
            perf: "Performance Improvements",
        };
        changelog.write("# " + version + " (" + currentDate() + ")" + os_1.EOL + "---" + os_1.EOL);
        releasePackage.commits.forEach(function (commit) {
            var message = commit.message;
            var type = message.type;
            if (!titles[type])
                return;
            if (message.breakingChanges) {
                sections.breaks.push(commit);
            }
            else {
                sections[type] = sections[type].concat(commit);
            }
        });
        Object.keys(titles).forEach(function (section) {
            var commits = sections[section];
            var title = titles[section];
            if (commits.length === 0)
                return;
            changelog.write(os_1.EOL + "## " + title + os_1.EOL + os_1.EOL);
            if (section === 'breaks') {
                commits.forEach(function (commit, i) {
                    changelog.write(i + 1 + ". " + commit.message.breakingChanges + os_1.EOL);
                    changelog.write("  - " + commit.message.raw.split(os_1.EOL)[0].trim() + " " +
                        ("" + linkToCommit(commit.hash, bugs.url)));
                    changelog.write(os_1.EOL);
                });
            }
            else {
                commits.forEach(function (commit) {
                    changelog.write("- " + commit.message.raw.split(os_1.EOL)[0].trim() + " " +
                        ("" + linkToCommit(commit.hash, bugs.url)));
                    changelog.write(os_1.EOL);
                });
            }
        });
        changelog.write(os_1.EOL);
        changelog.write(fileContents);
        changelog.on('finish', function () { return resolve(releasePackage); });
        changelog.on('end', function () { return resolve(releasePackage); });
        try {
            changelog.end();
        }
        catch (e) {
            reject(e);
        }
    });
}
function linkToCommit(hash, url) {
    return "[" + hash.substr(0, 8) + "](" + url.replace('/issues', '') + "/commits/" + hash + ")";
}
function currentDate() {
    var now = new Date();
    var pad = function (i) {
        return ('0' + i).substr(-2);
    };
    return now.getFullYear() + "-" +
        (pad(now.getMonth() + 1) + "-") +
        ("" + pad(now.getDate()));
}
exports.currentDate = currentDate;
//# sourceMappingURL=generateChangelog.js.map