"use strict";
var _1 = require("../../");
var typed_colors_1 = require("typed-colors");
var os_1 = require("os");
var bumpPackageVersions_1 = require("./bumpPackageVersions");
var generateChangelogs_1 = require("./generateChangelogs");
var getPackagesToUpdate_1 = require("./bumpPackageVersions/getPackagesToUpdate");
var getNewVersion_1 = require("./bumpPackageVersions/getNewVersion");
var splitVersion_1 = require("./bumpPackageVersions/splitVersion");
var helpers_1 = require("../../helpers");
var gitPushToReleaseBranch_1 = require("./gitPushToReleaseBranch");
var gitTags_1 = require("./gitTags");
var incrementName_1 = require("./bumpPackageVersions/incrementName");
var npmLogin_1 = require("./npmLogin");
var npmPublish_1 = require("./npmPublish");
var runTests_1 = require("./runTests");
var child_process_1 = require("child_process");
var switchToReleaseBranch_1 = require("./switchToReleaseBranch");
var _a = require('simple-spinner'), start = _a.start, stop = _a.stop, change_sequence = _a.change_sequence;
change_sequence([
    '    ',
    '•   ',
    '••  ',
    '••• ',
    '••••',
    'ᗧ•••',
    'ⵔ•••',
    ' ᗧ••',
    ' ⵔ••',
    '  ᗧ•',
    '  ⵔ•',
    '   ᗧ',
    '   ⵔ',
]);
var releaseDescription = _1.description('Automated package releases');
var checkFlag = _1.flag('boolean', _1.alias('check'), _1.description('Calculate releases to make'));
var comverFlag = _1.flag('boolean', _1.alias('comver'), _1.description('Release using Compatible versioning'));
var semverFlag = _1.flag('boolean', _1.alias('semver'), _1.description('Release using Semantic Versioning'));
var skipLoginFlag = _1.flag('boolean', _1.alias('skip-login'), _1.description('Avoid logging in to NPM'));
var releaseBranch = _1.flag('string', _1.alias('release-branch'), _1.description('Sets the git branch to push releases to'));
var skipTestsFlag = _1.flag('boolean', _1.alias('skip-tests'), _1.description('Avoid re-running tests'));
exports.plugin = _1.command(_1.alias('release'), releaseDescription, checkFlag, comverFlag, semverFlag, skipLoginFlag, releaseBranch, skipTestsFlag);
function isDirectoryClean(workingDir) {
    var isUnstagedChanges = child_process_1.spawnSync('git diff --exit-code', { cwd: workingDir }).status;
    var isStagedChanged = child_process_1.spawnSync('git diff --cached --exit-code', { cwd: workingDir }).status;
    return !(isUnstagedChanges || isStagedChanged);
}
_1.withCallback(exports.plugin, function (_a, io) {
    var config = _a.config, directory = _a.directory, options = _a.options;
    if (!isDirectoryClean(directory))
        return io.stdout.write("Git Directory is not clean" + os_1.EOL + os_1.EOL);
    var begin = options.check
        ? _1.changedPackages(directory)
        : switchToReleaseBranch_1.switchToReleaseBranch(options.releaseBranch, void 0, directory)
            .then(function () { return _1.changedPackages(directory); });
    return begin
        .then(function (affectedPackages) {
        if (Object.keys(affectedPackages).length === 0)
            throw "No packages currently require a new release";
        var method = options.comver ? 'comver' : 'semver';
        var header = generateHeader(config, affectedPackages, method);
        if (options.check) {
            stop();
            io.stdout.write(header + os_1.EOL + os_1.EOL);
            process.exit(0);
        }
        io.stdout.write(header);
        if (!options.skipTests) {
            io.stdout.write(os_1.EOL + typed_colors_1.bold("Running tests"));
            start();
        }
        return runTests_1.runTests(directory, options)
            .then(function () { stop(); io.stdout.write(os_1.EOL + os_1.EOL); })
            .then(function () { return bumpPackageVersions_1.bumpPackageVersions(config, io, options, affectedPackages); })
            .then(function (packages) {
            if (!options.skipLogin) {
                io.stdout.write('Logging in to NPM...' + os_1.EOL + os_1.EOL);
                return npmLogin_1.npmLogin(io, directory).then(function () { return packages; });
            }
            return packages;
        })
            .then(gitTags_1.gitTags(config, io))
            .then(generateChangelogs_1.generateChangelogs)
            .then(npmPublish_1.npmPublish(io))
            .then(function () { return gitPushToReleaseBranch_1.gitPushToReleaseBranch(options.releaseBranch, directory, io); });
    })
        .then(function () {
        stop();
        io.stdout.write(os_1.EOL);
    })
        .catch(function (e) {
        stop();
        if (typeof e.stderr === 'string') {
            io.stdout.write(e.stdout + os_1.EOL);
            io.stderr.write(e.stderr + os_1.EOL);
        }
        else {
            io.stderr.write((e.message || e) + os_1.EOL);
        }
        process.exit(1);
    });
});
var separator = '##----------------------------------------------------------------------------##';
function generateHeader(config, affectedPackages, method) {
    var packages = config.packages;
    var affectedPackageNames = Object.keys(affectedPackages);
    var packagesToUpdate = getPackagesToUpdate_1.getPackagesToUpdate(packages, affectedPackageNames);
    if (packagesToUpdate.length === 0)
        return separator + os_1.EOL + reportHeaderNegative() + os_1.EOL + separator + os_1.EOL;
    var message = separator + os_1.EOL + reportHeaderPositive();
    packagesToUpdate.forEach(function (releasePackage) {
        var name = releasePackage.name, pkg = releasePackage.pkg;
        var commits = affectedPackages[name].commits;
        var suggestedUpdate = helpers_1.getSuggestedUpdate(commits, method);
        var increment = incrementName_1.incrementName(suggestedUpdate);
        if (!increment)
            return;
        message += typed_colors_1.cyan(typed_colors_1.bold(name)) + " " +
            ("needs a new " + typed_colors_1.bold(increment.toUpperCase()) + " version released: ") +
            getSuggestedVersion(pkg.version, suggestedUpdate) + os_1.EOL;
        commits.forEach(function (commit) {
            var _a = commit.message, type = _a.type, scope = _a.scope, subject = _a.subject, breakingChanges = _a.breakingChanges;
            message += typed_colors_1.bold('  - ') + (type + "(" + scope + "): " + subject) +
                ("" + (breakingChanges && ' - ' + typed_colors_1.cyan('BREAKING CHANGE') || '')) + os_1.EOL;
        });
    });
    message += os_1.EOL + separator + os_1.EOL;
    return message;
}
function reportHeaderPositive() {
    return '                                ' + typed_colors_1.underline(typed_colors_1.bold('RELEASES TO DO')) + os_1.EOL + os_1.EOL +
        'We checked all packages and recent commits and discovered that' + '\n' +
        'you should release new versions for the following packages' + os_1.EOL + os_1.EOL;
}
function reportHeaderNegative() {
    return 'Nothing to release.' + os_1.EOL + os_1.EOL +
        'We checked all packages and recent commits and discovered that' + os_1.EOL +
        'you do not need to release any new version.';
}
function getSuggestedVersion(version, suggestedUpdate) {
    return getNewVersion_1.getNewVersion(splitVersion_1.splitVersion(version), suggestedUpdate);
}
//# sourceMappingURL=release.js.map